require 'paypal/masspay/version'
require 'paypal/masspay/response_parser'
require 'paypal/masspay/recipient'
require 'paypal/masspay/length_validator'
require 'paypal/masspay/configuration'

require 'faraday'
require 'ostruct'

module Paypal
  class Masspay
    SANDBOX_API_URL    = 'https://api-3t.sandbox.paypal.com'
    PRODUCTION_API_URL = 'https://api-3t.paypal.com'
    API_PATH           = '/nvp'
    API_VERSION        = '2.3'
    IDENTIFIER         = 'EmailAddress'
    METHOD             = 'MassPay'
    MAX_PER_REQUEST    = 250

    DEFAULT_CURRENCY   = 'USD'
    DEFAULT_SUBJECT    = 'Paypal Masspay'

    CURRENCY_MAX_LENGTH = 3
    SUBJECT_MAX_LENGTH  = 255

    attr_reader :errors

    def initialize(options = {}, &block)
      build_options(options)

      @success   = true
      @delivered = false
      @errors    = []

      yield(self) if block_given?
    end

    def add_recipient(recipient)
      @recipients << recipient
    end

    def deliver
      validate

      connection = create_connection

      @recipients.each_slice(MAX_PER_REQUEST) do |recipients_slice|
        response = connection.post(API_PATH, request_data(recipients_slice))
        check_response(response)
      end

      @delivered = true
    end

    def successful?
      @success
    end

    def delivered?
      @delivered
    end

    private

    def validate
      Paypal::Masspay::LengthValidator.validate(@currency, CURRENCY_MAX_LENGTH, :currency)
      Paypal::Masspay::LengthValidator.validate(@subject, SUBJECT_MAX_LENGTH, :subject)
    end

    def create_connection
      Faraday.new(:url => api_url) do |con|
        con.use(Faraday::Request::UrlEncoded)
        con.use(Faraday::Adapter::NetHttp)
      end
    end

    def api_url
      if Paypal::Masspay::Configuration.environment == :production
        PRODUCTION_API_URL
      else
        SANDBOX_API_URL
      end
    end

    def check_response(response)
      response = Paypal::Masspay::ResponseParser.new(response.body)
      if response.successful?
        @success = true && @success
      else
        @success = false
        @errors.concat(response.errors)
      end
    end

    def request_data(recipients)
      (default_options.merge(credentials).merge(build_payments(recipients))).delete_if { |k, v| v.nil? }
    end

    def build_options(options)
      @recipients  = options[:recipients] || []
      @currency    = options[:currency]   || DEFAULT_CURRENCY
      @subject     = options[:subject]    || DEFAULT_SUBJECT
    end

    def build_payments(recipients)
      Hash.new({}).tap do |output|
        recipients.each_with_index do |payment, index|
          output.merge!(payment.get_params(index))
        end
      end
    end

    def credentials
      {
        'USER'      => Paypal::Masspay::Configuration.username,
        'PWD'       => Paypal::Masspay::Configuration.password,
        'SIGNATURE' => Paypal::Masspay::Configuration.signature
      }
    end

    def default_options
      {
        'VERSION'      => API_VERSION,
        'METHOD'       => METHOD,
        'RECEIVERTYPE' => IDENTIFIER,
        'CURRENCYCODE' => @currency,
        'EMAILSUBJECT' => @subject
      }
    end
  end
end
